module Agilib
  class TokensController < Devise::SessionsController
    if Agilib.rails4?
      skip_before_action :verify_authenticity_token
    else
      skip_before_filter :verify_authenticity_token
    end

    respond_to :json

    def create
      login     = params[Agilib::TokenAuthenticatable.login_params[:login]]
      password  = params[Agilib::TokenAuthenticatable.login_params[:password]]

      if request.format != :json
        render :status=>406, :json => { :message => t('agilib.token_authenticatable.format_error', :default => "The request must be json") }
        return
      end

      if login.nil? or password.nil?
        render :status=>400, :json=>{ :message => t('agilib.token_authenticatable.request_login_and_password', :default => "The request must contain the user login and password.") }
        return
      end

      if User.respond_to? "find_by"
        @user = User.find_by(email: login.downcase)
      elsif User.respond_to? "find_by_email"
        @user = User.find_by_email(login.downcase)
      end

      if @user.nil?
        render :status=>401, :json=>{:message=>t('agilib.token_authenticatable.invalid_login_or_password', :default => "Invalid login or passoword.")}
        return
      end

      @user.ensure_authentication_token
      @user.save!

      if not @user.valid_password?(password)
        render :status=> 401, :json=>{:message=>t('agilib.token_authenticatable.invalid_login_or_password', :default => "Invalid login or passoword.")}
      else
        render :status=>200, :json=>{:user=> @user, :token => @user.authentication_token}
      end

    end

    def destroy
      token = params[Agilib::TokenAuthenticatable.auth_params[:user_token]]
      
      if User.respond_to? "find_by"
        @user = User.find_by(authentication_token: token)
      elsif User.respond_to? "find_by_authentication_token"
        @user = User.find_by_authentication_token(token)
      end

      if @user.nil?
        render :status=>404, :json=>{:message=>t('agilib.token_authenticatable.invalid_token', :default => "Invalid Token")}
      else
        @user.authentication_token = ""
        @user.save!

        sign_out @user
        
        render :status => 200, :json=>{:user=> @user, :token=>params[:token]}
      
      end
    end

      # GET /tokens/1/check.json
    def check
      token = params[Agilib::TokenAuthenticatable.auth_params[:user_token]]
      
      if User.respond_to? "find_by"
        @user = User.find_by(authentication_token: token)
      elsif User.respond_to? "find_by_authentication_token"
        @user = User.find_by_authentication_token(token)
      end
    
      if @user.nil?
        render :status=>404, :json=>{:message=>t('agilib.token_authenticatable.invalid_token', :default => "Invalid Token")}
        return
      else
        render :status=>200, :json=>{:user=> @user, :token => @user.authentication_token}
      end
    end

  end
end