# encoding: utf-8

gem 'rspec-rails', 				:group => [:development, :test]
gem 'factory_girl_rails', :group => [:development, :test]
gem "database_cleaner",   :group => :test
gem "email_spec",     		:group => :test
gem 'simplecov', 					:require => false, :group => :test
gem 'rspec_api_documentation', 	:group => [:development, :test]
gem 'json_spec', :group => [:development, :test]

templater.post_bundler do 

	rake 'db:migrate'
	rake 'db:test:prepare'

	spec_helper_path = 'spec/spec_helper.rb'

	## ===============================================================
	## RSPEC
	## ===============================================================
	
	say("\nIniciando configuração da Suíte de Testes... \n", :yellow)

	say("\nConfigurando RSpec...\n", :green)

	generate 'rspec:install'

	generators_config = <<-RUBY

		config.generators do |g|
		     g.test_framework :rspec, :fixture => true, :view_specs => false, :request_specs => false, :routing_specs => false, :controller_specs => false, :helper_specs => false
		     g.fixture_replacement :factory_girl, dir: 'spec/factories'
		     g.view_specs false
		     g.helper_specs false
		     g.controller_specs false
		 end

	RUBY

	
  gsub_file spec_helper_path, 'config.fixture_path = "#{::Rails.root}/spec/fixtures"', ''

	inject_into_file "config/application.rb", generators_config, :after => "class Application < Rails::Application"

	## ===============================================================
	## DATABASE CLEANER
	## ===============================================================
	
	say("\nConfigurando Database Cleaner...\n", :green)

	database_cleaner_config = <<-RUBY

	config.before(:suite) do
    DatabaseCleaner.strategy = :truncation
  end

  config.before(:each) do
    DatabaseCleaner.start
  end

  config.after(:each) do
    DatabaseCleaner.clean
	end

	RUBY


	inject_into_file spec_helper_path, database_cleaner_config, :after => 'config.order = "random"'


	## ===============================================================
	## FACTORY GIRLS
	## ===============================================================
	
	say("\nConfigurando FactoryGirl...\n", :green)

	inject_into_file spec_helper_path, "\n\n  config.include FactoryGirl::Syntax::Methods", :after => 'RSpec.configure do |config|'


	## ===============================================================
	## Email Spec
	## ===============================================================
	
	say("\nConfigurando EmailSpec...\n", :green)

	inject_into_file spec_helper_path, "\nrequire 'email_spec'", :after => "require 'rspec/rails'"

	## ===============================================================
	## SimpleCov
	## ===============================================================
	
	say("\nConfigurando SimpleCov...\n", :green)

	simple_cov_config = <<-TEXT
require 'simplecov'
SimpleCov.start
	TEXT

	inject_into_file spec_helper_path, simple_cov_config, :before => "ENV[\"RAILS_ENV\"] ||= 'test'"

	## ===============================================================
	## Paperclip
	## ===============================================================
	
	say("\nConfigurando Paperclip...\n", :green)

	inject_into_file spec_helper_path, "\nrequire 'paperclip/matchers'", :after => "require 'rspec/autorun'"


	## ===============================================================
	## Cancan
	## ===============================================================
	
	say("\nConfigurando Cancan...\n", :green)

	inject_into_file spec_helper_path, "\nrequire 'cancan/matchers'", :after => "require 'rspec/autorun'"



	## ===============================================================
	## USER SPECS
	## ===============================================================
	
	say("\nConfigurando Testes para User...\n", :green)

	remove_file "spec/models/user_spec.rb"
	create_file "spec/factories/users.rb", templater.load_snippet('users_factory','rspec')
	create_file "spec/models/user_spec.rb", templater.load_snippet('user_model_spec','rspec')



	## ===============================================================
	## RSPEC API DOCUMENTATION
	## ===============================================================
	
	say("\nConfigurando RSpec API Documentation...\n", :green)

	append_file spec_helper_path do 
		<<-TEXT
RspecApiDocumentation.configure do |config|
  config.format = [:json]
  config.keep_source_order = false
end
		TEXT
	end

	create_file "spec/acceptance/users_spec.rb", 	templater.load_snippet('users_acceptance_spec','rspec')
	create_file "spec/acceptance/tokens_spec.rb", templater.load_snippet('tokens_acceptance_spec','rspec')


	## ===============================================================
	## USER SPECS
	## ===============================================================
	
	say("\nConfigurando Testes para User...\n", :green)

	remove_file "spec/models/user_spec.rb"
	create_file "spec/models/user_spec.rb", templater.load_snippet('user_model_spec','rspec')


end