module Agilib
  module TokenAuthenticationHandlerMethods
    extend ActiveSupport::Concern

    # Please see https://gist.github.com/josevalim/fb706b1e933ef01e4fb6
    # before editing this file, the discussion is very interesting.

    included do
      private :authenticate_user_from_token!
      # This is our new function that comes before Devise's one
      before_filter :authenticate_user_from_token!
      # This is Devise's authentication
      before_filter :authenticate_user!
    end

    # For this example, we are simply using token authentication
    # via parameters. However, anyone could use Rails's token
    # authentication features to get the token from a header.
    def authenticate_user_from_token!
      param_user_token    = params[Agilib::TokenAuthenticatable.auth_params[:user_token]]
      param_email_token   = params[Agilib::TokenAuthenticatable.auth_params[:user_email]]

      # Set the authentication token params if not already present,
      # see http://stackoverflow.com/questions/11017348/rails-api-authentication-by-headers-token
      if user_token = param_user_token.blank? && request.headers["X-User-Token"]
        param_user_token = user_token
      end
      if user_email = params[Agilib::TokenAuthenticatable.auth_params[:user_email]].blank? && request.headers["X-User-Email"]
        param_email_token = user_email
      end

      user_email = param_email_token.presence
      # See https://github.com/ryanb/cancan/blob/1.6.10/lib/cancan/controller_resource.rb#L108-L111
      if User.respond_to? "find_by"
        user = user_email && User.find_by(email: user_email)
      elsif User.respond_to? "find_by_email"
        user = user_email && User.find_by_email(user_email)
      end

      # Notice how we use Devise.secure_compare to compare the token
      # in the database with the token given in the params, mitigating
      # timing attacks.
      if user && Devise.secure_compare(user.authentication_token, param_user_token)
        # Notice we are passing store false, so the user is not
        # actually stored in the session and a token is needed
        # for every request. If you want the token to work as a
        # sign in token, you can simply remove store: false.
        sign_in user, store: false
      end
    end
  end

  module TokenAuthenticationHandler
    extend ActiveSupport::Concern

    # I have insulated the methods into an additional module to avoid before_filters
    # to be applied by the `included` block before acts_as_token_authenticatable_handler was called.
    # See https://github.com/gonzalo-bulnes/simple_token_authenticatable/issues/8#issuecomment-31707201

    included do
      # nop
    end

    module ClassMethods
      def agilib_token_authenticatable(options = {})
        include Agilib::TokenAuthenticationHandlerMethods if Agilib.use_modules[:token_authenticatable].present?

        unless Agilib.use_modules[:token_authenticatable].present?
          # puts "Para usar a autenticacao por token, precisa ativar na configuracao: use_token_authenticatable = true"
        end
      end
    end
  end
end
if defined? ActionController::Base
  ActionController::Base.send :include, Agilib::TokenAuthenticationHandler
end